/*
 * Copyright (C) 2009 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef V8BindingState_h
#define V8BindingState_h

#include "GenericBinding.h"
#include "V8Binding.h"

namespace WebCore {

class Frame;

// Singleton implementation of State<V8Binding>.  Uses V8's global data
// structures to return information about relevant execution state.
template <>
class State<V8Binding> : public State<GenericBinding> {
public:
    // Singleton
    static State* Only();

    // Reports an error message (without delay) if the security check fails.
    static void immediatelyReportUnsafeAccessTo(Frame*);

    DOMWindow* activeWindow();
    DOMWindow* firstWindow();

    Frame* activeFrame();
    Frame* firstFrame();

    bool processingUserGesture();

    // FIXME: This should be shared in BindingSecurity
    bool allowsAccessFromFrame(Frame*);

private:
    explicit State() {}
    ~State();
};

typedef State<V8Binding> V8BindingState;

}

#endif // V8BindingState_h
