/*
 * Copyright 2010, The Android Open Source Project
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef PackageNotifier_h
#define PackageNotifier_h

#if ENABLE(APPLICATION_INSTALLED)

#include "PlatformString.h"
#include "Timer.h"

#include <wtf/HashSet.h>
#include <wtf/Noncopyable.h>
#include <wtf/text/StringHash.h>

namespace WebCore {

class PackageNotifier {
    WTF_MAKE_NONCOPYABLE(PackageNotifier);
public:
    PackageNotifier();

    typedef void (*Callback)();
    void setOnResultAvailable(Callback);

    void addPackageNames(const HashSet<String>& packageNames);
    void addPackageName(const String& packageName);
    void removePackageName(const String& packageName);

    void requestPackageResult();
    bool isPackageInstalled(const String& packageName);

private:
    void timerFired(Timer<PackageNotifier>*);

    Callback m_onResultAvailable;

    HashSet<String> m_packageNames;
    bool m_isInitialized;
    Timer<PackageNotifier> m_timer;
};

PackageNotifier& packageNotifier();

}

#endif

#endif // PackageNotifier_h
